import java.awt.*;
/**
  *
  * Klasse zur Simulation einer Spielposition beim HusBao-Spiel
  *
  * @version 1.0 vom 25.05.2010
  * @author Garmann
  */

public class Position {

  // Anfang Variablen
  private int amZug;
  private int[][] feld;
  private Graphics2D ausgabe;
  
  public static int[][] xpos = {{0,1,2,3,4,5,6,7,7,6,5,4,3,2,1,0},{7,6,5,4,3,2,1,0,0,1,2,3,4,5,6,7}};
  public static int[][] ypos = {{0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1},{3,3,3,3,3,3,3,3,2,2,2,2,2,2,2,2}};
  public static final int FAECHER = 16;

  // Ende Variablen

  public Position(Graphics2D ausgabe) {
    this.ausgabe = ausgabe;
    feld = new int[2][FAECHER];
    for (int spieler = 0; spieler < 2; spieler++) {
      for (int fach = 0; fach < FAECHER; fach++) {
        feld[spieler][fach] = 2;
      }
    }
    amZug = 0;
  }

  public Position(int amZug, Graphics2D ausgabe) {
    this(ausgabe);
    this.amZug = amZug;
  }
  
  public int getAmZug() {
    return amZug;
  }

  public void setAmZug(int amZug) {
    this.amZug = amZug;
  }
  
  public int getFeld(int fach, int spieler) {
    return feld[spieler][fach];
  }
  
  public void setFeld(int fach, int steine, int spieler) {
    feld[spieler][fach] = steine;
  }
  
  public void zugAusfuehren(int fach, int richtung) {
    int zuege = 0;
    while (zulaessig(fach) && !this.spielendeErreicht()) {
      int steine = feld[this.amZug][fach];
      feld[this.amZug][fach] = 0;
      for (int i = 0; i < steine; i++) {
        fach = fach + richtung;
        if (fach > FAECHER-1) fach = 0;
        if (fach < 0) fach = FAECHER-1;
        feld[this.amZug][fach]++;
      }
      if ((feld[this.amZug][fach] >= 2) && (fach > FAECHER/2-1)) {
        feld[this.amZug][fach] += feld[1-this.amZug][FAECHER + FAECHER / 2 - 1 - fach];
        feld[1-this.amZug][FAECHER + FAECHER / 2 - 1 - fach] = 0;
      }
      zuege++;
      //System.out.println("Zge: "+zuege);
      if(zuege > 500) {
        System.out.println("Zugberschreitung");
        System.out.println(this.toString());
        System.out.println("aktuelles Fach: "+fach+"   aktuelle Richtung: "+richtung);
        return;
        //System.exit(0);
      }
    }
    this.amZug = 1-this.amZug;
  }
  
  public Position folgePosition(int fach, int richtung) {
    Position neuePosition = new Position(this.ausgabe);
    for (int spieler = 0; spieler < 2; spieler++) {
      for (int i = 0; i < FAECHER; i++) {
        neuePosition.setFeld(i, this.feld[spieler][i], spieler);
      }
    }
    neuePosition.zugAusfuehren(fach, richtung);
    return neuePosition;
  }

  public boolean zulaessig(int fach) {
    return (feld[this.amZug][fach] >=2);
  }

  public boolean spielendeErreicht() {
    int[] summe = new int[2];
    for (int i = FAECHER / 2; i < FAECHER; i++) {
      summe[0] += feld[0][i];
      summe[1] += feld[1][i];
    }
    if (summe[0] == 0 || summe[1] == 0) {
      return true;
    }
    for (int i = 0; i < FAECHER; i++) {
      if (feld[this.amZug][i] >= 2) {
        return false;
      }
    }
    return true;
  }

  public void ausgabe() {
    ausgabe.setColor(Color.BLACK);
    ausgabe.drawRoundRect(20,20,600,270,20,20);
    ausgabe.drawLine(40,155,600,155);
    for (int spieler = 0; spieler < 2; spieler++) {
      for (int fach = 0; fach < FAECHER; fach++) {
        int x = xpos[spieler][fach]*70+60;
        int y = ypos[spieler][fach]*60+40;
        ausgabe.setColor(Color.WHITE);
        ausgabe.fillOval(x,y,50,50);
        ausgabe.setColor(Color.BLACK);
        ausgabe.drawOval(x,y,50,50);
        ausgabe.drawString(""+this.feld[spieler][fach],x+25,y+25);
      }
    }
    int[] steinsumme = new int[2];
    for (int fach = 0; fach < FAECHER; fach++) {
      steinsumme[0] += feld[0][fach];
      steinsumme[1] += feld[1][fach];
    }
    ausgabe.setColor(Color.WHITE);
    ausgabe.fillRoundRect(50,21,570,10,20,20);
    ausgabe.fillRoundRect(50,278,570,10,20,20);
    ausgabe.setColor(Color.BLACK);
    ausgabe.drawString("Anzahl Steine: "+steinsumme[0],50,31);
    ausgabe.drawString("Anzahl Steine: "+steinsumme[1],50,288);
    System.out.println(this.toString());
  }
  
  public String toString() {
    String result = "";
    result = result + " +-------------------------------+ \r\n";
    for (int fach = 0; fach < FAECHER/2; fach++) {
      result = result + " | "+feld[0][fach];
    }
    result = result + " | \r\n";
    for (int fach = FAECHER-1; fach > FAECHER/2-1; fach--) {
      result = result + " | "+feld[0][fach];
    }
    result = result + " | \r\n | ----------------------------- | \r\n";
    for (int fach = FAECHER/2; fach < FAECHER; fach++) {
      result = result + " | "+feld[1][fach];
    }
    result = result + " | \r\n";
    for (int fach = FAECHER/2-1; fach >= 0; fach--) {
      result = result + " | "+feld[1][fach];
    }
    result = result + " | \r\n +-------------------------------+ \r\n";
    return result;
  }
}

